const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
/*const HtmlWebpackInlineSourcePlugin=require('html-webpack-inline-source-plugin');*/
const CleanWebpackPlugin = require('clean-webpack-plugin') // 清空打包目录的插件
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const webpack = require('webpack');
const proxyHost = "http://192.168.2.241:5858";
// const proxyHost = "http://192.168.3.117:5858";//铁钢
module.exports = {
  entry: {
    index: path.resolve(__dirname, 'src/js', 'index.js'),
    qcScore:path.resolve(__dirname, 'src/js', 'qcScore.js'),
    login:path.resolve(__dirname, 'src/js', 'login.js'),
    console:path.resolve(__dirname, 'src/js', 'console.js'),
    moduleManager: path.resolve(__dirname, 'src/js', 'moduleManager.js'),
    itemManager: path.resolve(__dirname, 'src/js', 'itemManager.js'),
    qcList: path.resolve(__dirname, 'src/js', 'qcList.js'),
    vendor: 'lodash'// 多个页面所需的公共库文件，防止重复打包带入
  },
  output: {
    publicPath: '/', //这里要放的是静态资源CDN的地址
    path: path.resolve(__dirname, 'dist'),
    filename: 'js/[name].js'
  },
  resolve: {
    extensions: [".js", ".css", ".json"],
    alias: {} //配置别名可以加快webpack查找模块的速度
  },
  plugins: [//多入口的html文件用chunks这个参数来区分
    new HtmlWebpackPlugin({
      title: 'index',
      template: path.resolve(__dirname, 'src/html', 'index.html'),
      filename: 'index.html',
      chunks: ['index', 'vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new HtmlWebpackPlugin({
      title: 'console.html',
      template: path.resolve(__dirname, 'src/html', 'console.html'),
      filename: 'console.html',
      chunks: ['console', 'vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new HtmlWebpackPlugin({
      title: 'itemManager.html',
      template: path.resolve(__dirname, 'src/html', 'itemManager.html'),
      filename: 'itemManager.html',
      chunks: ['itemManager', 'vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new HtmlWebpackPlugin({
      title: 'moduleManager.html',
      template: path.resolve(__dirname, 'src/html', 'moduleManager.html'),
      filename: 'moduleManager.html',
      chunks: ['moduleManager', 'vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new HtmlWebpackPlugin({
      title: 'login.html',
      template: path.resolve(__dirname, 'src/html', 'login.html'),
      filename: 'login.html',
      chunks: ['login', 'vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new HtmlWebpackPlugin({
      title: 'qcList.html',
      template: path.resolve(__dirname, 'src/html', 'qcList.html'),
      filename: 'qcList.html',
      chunks: ['qcList', 'vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new HtmlWebpackPlugin({
      title: 'qcScore.html',
      template: path.resolve(__dirname, 'src/html', 'qcScore.html'),
      filename: 'qcScore.html',
      chunks: [ 'qcScore','vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new HtmlWebpackPlugin({
      title: 'statistics.html',
      template: path.resolve(__dirname, 'src/html', 'statistics.html'),
      filename: 'statistics.html',
      chunks: ['index', 'vendor', 'common'],
      hash: true, //防止缓存
      inject: true,
      minify: {
        removeAttributeQuotes: true, //压缩 去掉引号
        removeComments: true, //移除HTML中的注释
        collapseWhitespace: true //删除空白符与换行符
      }
    }),
    new CopyWebpackPlugin([
      {
        from:'src/resource',
        to:path.resolve(__dirname,'dist','resource'),
        flatten:true,   //false会拷贝原始文件夹路径
      }
    ]),
    new MiniCssExtractPlugin({
      filename: 'css/[name].css',
      chunkFilename: '[id].css'
    }),
    new webpack.HotModuleReplacementPlugin(),
    new CleanWebpackPlugin()
  ],
  optimization: { //webpack4.x的最新优化配置项，用于提取公共代码
    minimizer: [
      new UglifyJsPlugin({
        uglifyOptions: {
          ie8: true,
          compress: {
            properties: false,
            warnings: false
          },
          mangle: {
            screw_ie8: false,
            except: ['e']
          },
          output: {
              beautify: true
          },
          sourceMap: false
        }
      })
    ],
    splitChunks: {
      cacheGroups: {
        commons: {
          chunks: "initial",
          name: "common",
          minChunks: 2,
          maxInitialRequests: 5, // The default limit is too small to showcase the effect
          minSize: 0, // This is example is too small to create commons chunks
          reuseExistingChunk: true // 可设置是否重用该chunk
        }
      }
    }
  },
      
  module: {
    noParse: /WdatePicker/,
    rules: [
      {
          test: /.js$/,
          enforce: 'post', 
          loader: 'es3ify-loader'
      },
      {
        test: /\.m?js$/,
        exclude: /(node_modules|bower_components)/,
        use: {
          loader: 'babel-loader',
          options: {
            presets:['@babel/preset-env']
          }
        }
      },
      {
        test: /\.css$/,
        use: [{
            loader: MiniCssExtractPlugin.loader
          },
          'css-loader'
        ]
      },
      {
        test: /\.less$/,
        use: [{
            loader: MiniCssExtractPlugin.loader
          },
          'css-loader', 'less-loader'
        ]
      },
      {
        test:/\.(png|gif|jpg|jpeg|svg|eot|ttf|woff|woff2)$/,
        use:[{
          loader:'url-loader',
          options:{
            limit:10240,
            esModule:false,
            name:'[name]_[hash:6].[ext]',
            outputPath:'images/'
          }
        }],
        exclude:/node_modules/
      },{
        test:/.html$/,
        use:'html-withimg-loader'
      }
    ]
  },
  // devtool: 'cheap-module-eval-source-map', //开发环境cheap-module-eval-source-map   //生产环境cheap-module-source-map
  mode: 'development',
  devServer: {
    contentBase: "./dist", //静态文件根目录
    proxy: {
      '/': proxyHost
    },
    hot: true,
    openPage:'index.html'
    
  }
}